@@ -108,7 +108,8 @@ TECH_CLASSES = [
     "Python11",
     "Saipan",
     "PyramidPlus",
-    "Hbpl1",
+    "Kapan",
+    "MimasTDR",
 ]
 
 TECH_CLASSES.sort()
@@ -165,7 +166,8 @@ TECH_CLASS_PDLS = {
     "Copperhead"   : 'pcl3',
     "Copperhead12"   : 'pcl3',
     "Ampere"	   : 'pcl3',
-    "Hbpl1"         : 'hbpl1'
+    "Kapan"         : 'pcl3',
+    "MimasTDR"      : 'pcl3'
 }
 
 PDL_TYPE_PCL = 0  # less preferred
@@ -220,7 +222,10 @@ TECH_SUBCLASSES = [
     "Trim",
     "4800x1200dpi",
     "Advanced",
-    "AutoDuplex"
+    "Mono",
+    "Color",
+    "AutoDuplex",
+    "K10"
 ]
 
 TECH_SUBCLASSES.sort()
