@@ -54,6 +54,7 @@
 #include "hpip.h"
 #include "hpcupsfax.h"
 #include "bug.h"
+#include "utils.h"
 using namespace std;
 
 int    fax_encoding = RASTER_MMR;
@@ -76,6 +77,7 @@ char hpFileName[64] ;
 // GrayLevel = (5/16)R + (9/16)G + (2/16)B
 #define RGB2BW(r, g, b) (BYTE) (((r << 2) + r + (g << 3) + g + (b << 1)) >> 4)
 
+
 void RGB2Gray (BYTE *pRGBData, int iNumPixels, BYTE *pGData)
 {
     int     i;
@@ -420,7 +422,7 @@ BUGOUT:
  * Reading from stdin into a temp file
  * Getting the final file with HPLIP file and page headers
  */
-int ProcessTiffData(int fromFD, int toFD)
+int ProcessTiffData(int fromFD, int toFD, char* user_name)
 {
     BYTE      *p;
     int       fdTiff;
@@ -441,20 +443,16 @@ int ProcessTiffData(int fromFD, int toFD)
     int bytes_read = 0;
     char hpTiffFileName[64];
     long input_file_size = 0;
-    snprintf(hpTiffFileName,sizeof(hpTiffFileName), "%s/hpliptiffXXXXXX","/var/log/hp/tmp");
+    FILE* pFilePtrFax;
+    snprintf(hpTiffFileName,sizeof(hpTiffFileName), "%s/hp_%s_fax_tiffXXXXXX",CUPS_TMP_DIR,user_name);
 
 
-    fdTiff = mkstemp (hpTiffFileName);
+//    fdTiff = mkstemp (hpTiffFileName);
+    fdTiff = createTempFile(hpTiffFileName, &pFilePtrFax);
     if (fdTiff < 0)
     {
-//        DBG ("Warning: Unable to open Fax output file - %s for writing, so trying another location\n", hpTiffFileName);
-//        strncpy(hpTiffFileName,"/var/log/hp/hpliptiffXXXXXX",sizeof(hpTiffFileName));
-//        fdTiff = mkstemp (hpTiffFileName);
-//        if (fdTiff < 0)
-        {
-            BUG("ERROR: Unable to open Fax output file - %s for writing\n", hpTiffFileName);
-            return 1;
-        }
+        BUG("ERROR: Unable to open Fax output file - %s for writing\n", hpTiffFileName);
+        return 1;
     }
 
     memset (szFileHeader, 0, sizeof (szFileHeader));
@@ -648,7 +646,7 @@ int main (int argc, char **argv)
     int                 fd = 0;
     int                 fdFax = -1;
     int i = 0;
-    FILE                *fdTiff;
+    FILE                *pFilePtrFax;
     cups_raster_t       *cups_raster;
     ppd_file_t          *ppd;
     ppd_attr_t          *attr;
@@ -679,20 +677,17 @@ int main (int argc, char **argv)
          i++;
     }
 
-    snprintf(hpFileName,sizeof(hpFileName),"%s/hplipfaxLog_XXXXXX","/var/log/hp/tmp");
+    snprintf(hpFileName,sizeof(hpFileName),"%s/hp_%s_fax_Log_XXXXXX",CUPS_TMP_DIR, argv[2]);
 
-    fdFax = mkstemp (hpFileName);
+//    fdFax = mkstemp (hpFileName);
+    fdFax = createTempFile(hpFileName, &pFilePtrFax);
     if (fdFax < 0)
     {
-//        DBG ("Warning: Unable to open Fax output file - %s for writing,so trying another location.\n", hpFileName);
-//        strncpy(hpFileName,"/var/log/hp/hplipfaxLog_XXXXXX",sizeof(hpFileName));
-//        fdFax = mkstemp (hpFileName);
-//        if (fdFax < 0)
-        {
-            BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpFileName);
-            return 1;
-        }
+         BUG ("ERROR: Unable to open Fax output file - %s for writing\n", hpFileName);
+         return 1;
     }
+    else
+        chmod(hpFileName, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP );
 
     /*********** MAIN ***********/
 
@@ -759,7 +754,7 @@ int main (int argc, char **argv)
 
     if (fax_encoding == RASTER_TIFF)
     {
-        status = ProcessTiffData(fd, fdFax);
+        status = ProcessTiffData(fd, fdFax, argv[2]);
     } else {
        cups_raster = cupsRasterOpen (fd, CUPS_RASTER_READ);
        if (cups_raster == NULL)
