@@ -42,6 +42,7 @@
 #include "ijs_server.h"
 #include "hpijs.h"
 #include "services.h"
+#include "utils.h"
 
 extern void SendDbusMessage (const char *dev, const char *printer, int code, 
                              const char *username, const int jobid, const char *title);
@@ -71,7 +72,7 @@ int bug(const char *fmt, ...)
 }
 #endif
 
-void setLogLevel(UXServices *pSS)
+void setLogLevel(UXServices *pSS, char*user_name)
 {
     FILE    *fp;
     char    str[258];
@@ -97,9 +98,10 @@ void setLogLevel(UXServices *pSS)
     if (pSS->m_iLogLevel & SAVE_PCL_FILE)
     {
         char    szFileName[64];
-	snprintf (szFileName,sizeof(szFileName), "/var/log/hp/tmp/hpijs_%d.out", getpid());
+        snprintf (szFileName,sizeof(szFileName), "%s/hp_%s_ijs_%d_XXXXXX", CUPS_TMP_DIR, user_name,  getpid());
+        createTempFile(szFileName, &pSS->outfp);
 
-	pSS->outfp = fopen (szFileName, "w");
+//	pSS->outfp = fopen (szFileName, "w");
 	if (pSS->outfp)
 	{
 	    chmod (szFileName, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
@@ -586,7 +588,8 @@ int main (int argc, char *argv[], char *evenp[])
    char *raster = NULL, *k_raster = NULL;
    int status = EXIT_FAILURE;
    int ret, n, i, kn=0, width, k_width;
-
+   char user_name[32]={0,};
+        
    openlog("hpijs", LOG_PID,  LOG_DAEMON);
 
    if (argc > 1)
@@ -600,6 +603,9 @@ int main (int argc, char *argv[], char *evenp[])
       }
    }
 
+   if (argc > 2)
+        strncpy(user_name, argv[2], sizeof(user_name));
+
 #ifdef HAVE_LIBHPIP
    char *pDev;
    if ((pDev = getenv ("DEVICE_URI")) &&
@@ -623,11 +629,11 @@ int main (int argc, char *argv[], char *evenp[])
       goto BUGOUT;
    }
 
-   setLogLevel(pSS);
+   setLogLevel(pSS, user_name);
 
 #ifdef CAPTURE
    char szCapOutFile[64];
-   snprintf(szCapOutFile, sizeof(szCapOutFile),"%s/capout_XXXXXX","/var/log/hp/tmp");
+   snprintf(szCapOutFile, sizeof(szCapOutFile),"%s/hp_%s_ijs_capout_XXXXXX",CUPS_TMP_DIR, user_name);
    if ((pSS->InitScript(szCapOutFile, TRUE)) != NO_ERROR)
       BUG("unable to init capture");
 #endif
